@@ -18,6 +18,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#include <assert.h>
 #include "wine/debug.h"
 
 #define COBJMACROS
@@ -208,7 +209,7 @@ HRESULT d3drm_device_init(struct d3drm_device *device, UINT version, IDirectDraw
     return hr;
 }
 
-HRESULT d3drm_device_set_ddraw_device_d3d(struct d3drm_device *device, IDirect3D *d3d, IDirect3DDevice *d3d_device)
+static HRESULT d3drm_device_set_ddraw_device_d3d(struct d3drm_device *device, IDirect3D *d3d, IDirect3DDevice *d3d_device)
 {
     IDirectDraw *ddraw;
     IDirectDrawSurface *surface;
@@ -1617,6 +1618,15 @@ static const struct IDirect3DRMWinDeviceVtbl d3drm_device_win_vtbl =
     d3drm_device_win_HandleActivate,
 };
 
+struct d3drm_device *unsafe_impl_from_IDirect3DRMDevice3(IDirect3DRMDevice3 *iface)
+{
+    if (!iface)
+        return NULL;
+    assert(iface->lpVtbl == &d3drm_device3_vtbl);
+
+    return impl_from_IDirect3DRMDevice3(iface);
+}
+
 HRESULT d3drm_device_create(struct d3drm_device **device, IDirect3DRM *d3drm)
 {
     struct d3drm_device *object;
