@@ -29,11 +29,9 @@
 #include <stdarg.h>
 #include <stdio.h>
 
-#define NONAMELESSUNION
-#define NONAMELESSSTRUCT
-
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
+#define NONAMELESSUNION
 #include "windef.h"
 #include "winbase.h"
 #include "winerror.h"
@@ -59,8 +57,9 @@ static inline BOOL is_version_nt(void)
 HANDLE get_BaseNamedObjects_handle(void)
 {
     static HANDLE handle = NULL;
-    static const WCHAR basenameW[] =
-        {'\\','B','a','s','e','N','a','m','e','d','O','b','j','e','c','t','s',0};
+    static const WCHAR basenameW[] = {'\\','S','e','s','s','i','o','n','s','\\','%','u',
+                                      '\\','B','a','s','e','N','a','m','e','d','O','b','j','e','c','t','s',0};
+    WCHAR buffer[64];
     UNICODE_STRING str;
     OBJECT_ATTRIBUTES attr;
 
@@ -68,7 +67,8 @@ HANDLE get_BaseNamedObjects_handle(void)
     {
         HANDLE dir;
 
-        RtlInitUnicodeString(&str, basenameW);
+        sprintfW( buffer, basenameW, NtCurrentTeb()->Peb->SessionId );
+        RtlInitUnicodeString( &str, buffer );
         InitializeObjectAttributes(&attr, &str, 0, 0, NULL);
         NtOpenDirectoryObject(&dir, DIRECTORY_CREATE_OBJECT|DIRECTORY_TRAVERSE,
                               &attr);
@@ -1299,6 +1299,15 @@ BOOL WINAPI ChangeTimerQueueTimer( HANDLE TimerQueue, HANDLE Timer,
     return TRUE;
 }
 
+/***********************************************************************
+ *           CancelTimerQueueTimer    (KERNEL32.@)
+ */
+BOOL WINAPI CancelTimerQueueTimer(HANDLE queue, HANDLE timer)
+{
+    FIXME("stub: %p %p\n", queue, timer);
+    return FALSE;
+}
+
 /***********************************************************************
  *           DeleteTimerQueueTimer  (KERNEL32.@)
  *
@@ -1540,7 +1549,7 @@ BOOL WINAPI WaitNamedPipeW (LPCWSTR name, DWORD nTimeOut)
     pipe_dev_name.Length = sizeof(leadin);
     pipe_dev_name.MaximumLength = sizeof(leadin);
     InitializeObjectAttributes(&attr,&pipe_dev_name, OBJ_CASE_INSENSITIVE, NULL, NULL);
-    status = NtOpenFile( &pipe_dev, FILE_READ_ATTRIBUTES, &attr,
+    status = NtOpenFile( &pipe_dev, FILE_READ_ATTRIBUTES | SYNCHRONIZE, &attr,
                          &iosb, FILE_SHARE_READ | FILE_SHARE_WRITE,
                          FILE_SYNCHRONOUS_IO_NONALERT);
     if (status != ERROR_SUCCESS)
