@@ -92,6 +92,8 @@ struct ScriptHost {
     struct list entry;
 };
 
+static ScriptHost *get_elem_script_host(HTMLInnerWindow*,HTMLScriptElement*);
+
 static void set_script_prop(ScriptHost *script_host, DWORD property, VARIANT *val)
 {
     IActiveScriptProperty *script_prop;
@@ -140,7 +142,7 @@ static BOOL init_script_engine(ScriptHost *script_host)
 
     hres = IActiveScript_QueryInterface(script_host->script, &IID_IActiveScriptParse, (void**)&script_host->parse);
     if(FAILED(hres)) {
-        WARN("Could not get IActiveScriptHost: %08x\n", hres);
+        WARN("Could not get IActiveScriptParse: %08x\n", hres);
         return FALSE;
     }
 
@@ -735,7 +737,7 @@ static void fire_readystatechange_proc(task_t *_task)
         return;
 
     task->elem->pending_readystatechange_event = FALSE;
-    fire_event(task->elem->element.node.doc, EVENTID_READYSTATECHANGE, FALSE, task->elem->element.node.nsnode, NULL, NULL);
+    fire_event(task->elem->element.node.doc, EVENTID_READYSTATECHANGE, FALSE, &task->elem->element.node, NULL, NULL);
 }
 
 static void fire_readystatechange_task_destr(task_t *_task)
@@ -771,7 +773,7 @@ static void set_script_elem_readystate(HTMLScriptElement *script_elem, READYSTAT
         }else {
             script_elem->pending_readystatechange_event = FALSE;
             fire_event(script_elem->element.node.doc, EVENTID_READYSTATECHANGE, FALSE,
-                    script_elem->element.node.nsnode, NULL, NULL);
+                    &script_elem->element.node, NULL, NULL);
         }
     }
 }
@@ -784,8 +786,6 @@ static void parse_elem_text(ScriptHost *script_host, HTMLScriptElement *script_e
 
     TRACE("%s\n", debugstr_w(text));
 
-    set_script_elem_readystate(script_elem, READYSTATE_INTERACTIVE);
-
     VariantInit(&var);
     memset(&excepinfo, 0, sizeof(excepinfo));
     TRACE(">>>\n");
@@ -796,7 +796,6 @@ static void parse_elem_text(ScriptHost *script_host, HTMLScriptElement *script_e
         TRACE("<<<\n");
     else
         WARN("<<< %08x\n", hres);
-
 }
 
 typedef struct {
@@ -807,9 +806,111 @@ typedef struct {
 
     DWORD size;
     char *buf;
-    HRESULT hres;
 } ScriptBSC;
 
+static HRESULT get_binding_text(ScriptBSC *bsc, WCHAR **ret)
+{
+    UINT cp = CP_UTF8;
+    WCHAR *text;
+
+    if(!bsc->bsc.readed) {
+        text = heap_alloc(sizeof(WCHAR));
+        if(!text)
+            return E_OUTOFMEMORY;
+        *text = 0;
+        *ret = text;
+        return S_OK;
+    }
+
+    switch(bsc->bsc.bom) {
+    case BOM_UTF16:
+        if(bsc->bsc.readed % sizeof(WCHAR)) {
+            FIXME("The buffer is not a valid utf16 string\n");
+            return E_FAIL;
+        }
+
+        text = heap_alloc(bsc->bsc.readed+sizeof(WCHAR));
+        if(!text)
+            return E_OUTOFMEMORY;
+
+        memcpy(text, bsc->buf, bsc->bsc.readed);
+        text[bsc->bsc.readed/sizeof(WCHAR)] = 0;
+        break;
+
+    default:
+        /* FIXME: Try to use charset from HTTP headers first */
+        cp = get_document_charset(bsc->script_elem->element.node.doc);
+        /* fall through */
+    case BOM_UTF8: {
+        DWORD len;
+
+        len = MultiByteToWideChar(cp, 0, bsc->buf, bsc->bsc.readed, NULL, 0);
+        text = heap_alloc((len+1)*sizeof(WCHAR));
+        if(!text)
+            return E_OUTOFMEMORY;
+
+        MultiByteToWideChar(cp, 0, bsc->buf, bsc->bsc.readed, text, len);
+        text[len] = 0;
+    }
+    }
+
+    *ret = text;
+    return S_OK;
+}
+
+static void script_file_available(ScriptBSC *bsc)
+{
+    HTMLScriptElement *script_elem = bsc->script_elem;
+    HTMLInnerWindow *window = bsc->bsc.window;
+    ScriptHost *script_host;
+    nsIDOMNode *parent;
+    nsresult nsres;
+    HRESULT hres;
+
+    assert(window != NULL);
+
+    hres = get_binding_text(bsc, &script_elem->src_text);
+    if(FAILED(hres))
+        return;
+
+    script_host = get_elem_script_host(window, script_elem);
+    if(!script_host)
+        return;
+
+    if(window->parser_callback_cnt) {
+        script_queue_entry_t *queue;
+
+        TRACE("Adding to queue\n");
+
+        queue = heap_alloc(sizeof(*queue));
+        if(!queue)
+            return;
+
+        IHTMLScriptElement_AddRef(&script_elem->IHTMLScriptElement_iface);
+        queue->script = script_elem;
+
+        list_add_tail(&window->script_queue, &queue->entry);
+        return;
+    }
+
+    nsres = nsIDOMHTMLElement_GetParentNode(script_elem->element.nselem, &parent);
+    if(NS_FAILED(nsres) || !parent) {
+        TRACE("No parent, not executing\n");
+        script_elem->parse_on_bind = TRUE;
+        return;
+    }
+
+    nsIDOMNode_Release(parent);
+
+    script_host = get_elem_script_host(window, script_elem);
+    if(!script_host)
+        return;
+
+    script_elem->parsed = TRUE;
+    if(script_host->parse)
+        parse_elem_text(script_host, script_elem, script_elem->src_text);
+}
+
 static inline ScriptBSC *impl_from_BSCallback(BSCallback *iface)
 {
     return CONTAINING_RECORD(iface, ScriptBSC, bsc);
@@ -837,6 +938,8 @@ static HRESULT ScriptBSC_start_binding(BSCallback *bsc)
 {
     ScriptBSC *This = impl_from_BSCallback(bsc);
 
+    This->script_elem->binding = &This->bsc;
+
     /* FIXME: We should find a better to decide if 'loading' state is supposed to be used by the protocol. */
     if(This->scheme == URL_SCHEME_HTTPS || This->scheme == URL_SCHEME_HTTP)
         set_script_elem_readystate(This->script_elem, READYSTATE_LOADING);
@@ -848,11 +951,17 @@ static HRESULT ScriptBSC_stop_binding(BSCallback *bsc, HRESULT result)
 {
     ScriptBSC *This = impl_from_BSCallback(bsc);
 
-    This->hres = result;
+    if(SUCCEEDED(result) && !This->script_elem)
+        result = E_UNEXPECTED;
+
+    assert(FAILED(result) || This->script_elem->binding == &This->bsc);
+    This->script_elem->binding = NULL;
+
+    if(This->script_elem->readystate == READYSTATE_LOADING)
+        set_script_elem_readystate(This->script_elem, READYSTATE_LOADED);
 
     if(SUCCEEDED(result)) {
-        if(This->script_elem->readystate == READYSTATE_LOADING)
-            set_script_elem_readystate(This->script_elem, READYSTATE_LOADED);
+        script_file_available(This);
     }else {
         FIXME("binding failed %08x\n", result);
         heap_free(This->buf);
@@ -922,14 +1031,30 @@ static const BSCallbackVtbl ScriptBSCVtbl = {
 };
 
 
-static HRESULT bind_script_to_text(HTMLInnerWindow *window, IUri *uri, HTMLScriptElement *script_elem, WCHAR **ret)
+HRESULT load_script(HTMLScriptElement *script_elem, const WCHAR *src, BOOL async)
 {
-    UINT cp = CP_UTF8;
+    HTMLInnerWindow *window;
     ScriptBSC *bsc;
     IMoniker *mon;
-    WCHAR *text;
+    IUri *uri;
     HRESULT hres;
 
+    static const WCHAR wine_schemaW[] = {'w','i','n','e',':'};
+
+    if(strlenW(src) > sizeof(wine_schemaW)/sizeof(WCHAR) && !memcmp(src, wine_schemaW, sizeof(wine_schemaW)))
+        src += sizeof(wine_schemaW)/sizeof(WCHAR);
+
+    TRACE("(%p %s %x)\n", script_elem, debugstr_w(src), async);
+
+    if(!script_elem->element.node.doc || !(window = script_elem->element.node.doc->window)) {
+        ERR("no window\n");
+        return E_UNEXPECTED;
+    }
+
+    hres = create_uri(src, 0, &uri);
+    if(FAILED(hres))
+        return hres;
+
     hres = CreateURLMonikerEx2(NULL, uri, &mon, URL_MK_UNIFORM);
     if(FAILED(hres))
         return hres;
@@ -940,9 +1065,8 @@ static HRESULT bind_script_to_text(HTMLInnerWindow *window, IUri *uri, HTMLScrip
         return E_OUTOFMEMORY;
     }
 
-    init_bscallback(&bsc->bsc, &ScriptBSCVtbl, mon, 0);
+    init_bscallback(&bsc->bsc, &ScriptBSCVtbl, mon, async ? BINDF_ASYNCHRONOUS | BINDF_ASYNCSTORAGE | BINDF_PULLDATA : 0);
     IMoniker_Release(mon);
-    bsc->hres = E_FAIL;
 
     hres = IUri_GetScheme(uri, &bsc->scheme);
     if(FAILED(hres))
@@ -952,86 +1076,9 @@ static HRESULT bind_script_to_text(HTMLInnerWindow *window, IUri *uri, HTMLScrip
     bsc->script_elem = script_elem;
 
     hres = start_binding(window, &bsc->bsc, NULL);
-    if(SUCCEEDED(hres))
-        hres = bsc->hres;
-    if(FAILED(hres)) {
-        IBindStatusCallback_Release(&bsc->bsc.IBindStatusCallback_iface);
-        return hres;
-    }
-
-    if(!bsc->bsc.readed) {
-        *ret = NULL;
-        return S_OK;
-    }
-
-    switch(bsc->bsc.bom) {
-    case BOM_UTF16:
-        if(bsc->bsc.readed % sizeof(WCHAR)) {
-            FIXME("The buffer is not a valid utf16 string\n");
-            hres = E_FAIL;
-            break;
-        }
-
-        text = heap_alloc(bsc->bsc.readed+sizeof(WCHAR));
-        if(!text) {
-            hres = E_OUTOFMEMORY;
-            break;
-        }
-
-        memcpy(text, bsc->buf, bsc->bsc.readed);
-        text[bsc->bsc.readed/sizeof(WCHAR)] = 0;
-        break;
-
-    default:
-        /* FIXME: Try to use charset from HTTP headers first */
-        cp = get_document_charset(window->doc);
-        /* fall through */
-    case BOM_UTF8: {
-        DWORD len;
-
-        len = MultiByteToWideChar(cp, 0, bsc->buf, bsc->bsc.readed, NULL, 0);
-        text = heap_alloc((len+1)*sizeof(WCHAR));
-        if(!text) {
-            hres = E_OUTOFMEMORY;
-            break;
-        }
-
-        MultiByteToWideChar(cp, 0, bsc->buf, bsc->bsc.readed, text, len);
-        text[len] = 0;
-    }
-    }
 
     IBindStatusCallback_Release(&bsc->bsc.IBindStatusCallback_iface);
-    if(FAILED(hres))
-        return hres;
-
-    *ret = text;
-    return S_OK;
-}
-
-static void parse_extern_script(ScriptHost *script_host, HTMLScriptElement *script_elem, LPCWSTR src)
-{
-    WCHAR *text;
-    IUri *uri;
-    HRESULT hres;
-
-    static const WCHAR wine_schemaW[] = {'w','i','n','e',':'};
-
-    if(strlenW(src) > sizeof(wine_schemaW)/sizeof(WCHAR) && !memcmp(src, wine_schemaW, sizeof(wine_schemaW)))
-        src += sizeof(wine_schemaW)/sizeof(WCHAR);
-
-    hres = create_uri(src, 0, &uri);
-    if(FAILED(hres))
-        return;
-
-    hres = bind_script_to_text(script_host->window, uri, script_elem, &text);
-    IUri_Release(uri);
-    if(FAILED(hres) || !text)
-        return;
-
-    parse_elem_text(script_host, script_elem, text);
-
-    heap_free(text);
+    return hres;
 }
 
 static void parse_inline_script(ScriptHost *script_host, HTMLScriptElement *script_elem)
@@ -1044,6 +1091,8 @@ static void parse_inline_script(ScriptHost *script_host, HTMLScriptElement *scri
     nsres = nsIDOMHTMLScriptElement_GetText(script_elem->nsscript, &text_str);
     nsAString_GetData(&text_str, &text);
 
+    set_script_elem_readystate(script_elem, READYSTATE_INTERACTIVE);
+
     if(NS_FAILED(nsres)) {
         ERR("GetText failed: %08x\n", nsres);
     }else if(*text) {
@@ -1053,9 +1102,10 @@ static void parse_inline_script(ScriptHost *script_host, HTMLScriptElement *scri
     nsAString_Finish(&text_str);
 }
 
-static void parse_script_elem(ScriptHost *script_host, HTMLScriptElement *script_elem)
+static BOOL parse_script_elem(ScriptHost *script_host, HTMLScriptElement *script_elem)
 {
     nsAString src_str, event_str;
+    BOOL is_complete = FALSE;
     const PRUnichar *src;
     nsresult nsres;
 
@@ -1068,7 +1118,7 @@ static void parse_script_elem(ScriptHost *script_host, HTMLScriptElement *script
         if(*event) {
             TRACE("deferring event %s script evaluation\n", debugstr_w(event));
             nsAString_Finish(&event_str);
-            return;
+            return FALSE;
         }
     }else {
         ERR("GetEvent failed: %08x\n", nsres);
@@ -1082,15 +1132,16 @@ static void parse_script_elem(ScriptHost *script_host, HTMLScriptElement *script
     if(NS_FAILED(nsres)) {
         ERR("GetSrc failed: %08x\n", nsres);
     }else if(*src) {
-        script_elem->parsed = TRUE;
-        parse_extern_script(script_host, script_elem, src);
+        load_script(script_elem, src, FALSE);
+        is_complete = script_elem->parsed;
     }else {
         parse_inline_script(script_host, script_elem);
+        is_complete = TRUE;
     }
 
     nsAString_Finish(&src_str);
 
-    set_script_elem_readystate(script_elem, READYSTATE_COMPLETE);
+    return is_complete;
 }
 
 static GUID get_default_script_guid(HTMLInnerWindow *window)
@@ -1210,16 +1261,29 @@ static ScriptHost *get_elem_script_host(HTMLInnerWindow *window, HTMLScriptEleme
     return get_script_host(window, &guid);
 }
 
-void doc_insert_script(HTMLInnerWindow *window, HTMLScriptElement *script_elem)
+void doc_insert_script(HTMLInnerWindow *window, HTMLScriptElement *script_elem, BOOL from_parser)
 {
     ScriptHost *script_host;
+    BOOL is_complete = FALSE;
 
     script_host = get_elem_script_host(window, script_elem);
     if(!script_host)
         return;
 
-    if(script_host->parse)
-        parse_script_elem(script_host, script_elem);
+    if(script_host->parse) {
+        if(script_elem->src_text) {
+            if(from_parser)
+                set_script_elem_readystate(script_elem, READYSTATE_INTERACTIVE);
+            script_elem->parsed = TRUE;
+            parse_elem_text(script_host, script_elem, script_elem->src_text);
+            is_complete = TRUE;
+        }else if(!script_elem->binding) {
+            is_complete = parse_script_elem(script_host, script_elem);
+        }
+    }
+
+    if(is_complete)
+        set_script_elem_readystate(script_elem, READYSTATE_COMPLETE);
 }
 
 IDispatch *script_parse_event(HTMLInnerWindow *window, LPCWSTR text)
