@@ -574,16 +574,6 @@ int __thiscall MSVCRT_type_info_before(type_info * _this, const type_info * rhs)
     return ret;
 }
 
-/******************************************************************
- *		__std_type_info_compare (MSVCRT.@)
- */
-int CDECL MSVCRT_type_info_compare(type_info * _this, const type_info * rhs)
-{
-    int ret = strcmp(_this->mangled + 1, rhs->mangled + 1);
-    TRACE("(%p %p) returning %d\n", _this, rhs, ret);
-    return ret;
-}
-
 /******************************************************************
  *		??1type_info@@UAE@XZ (MSVCRT.@)
  */
@@ -1500,7 +1490,102 @@ void __cdecl __ExceptionPtrCurrentException(exception_ptr *ep)
 }
 #endif
 
+/*********************************************************************
+ * ?__ExceptionPtrToBool@@YA_NPBX@Z
+ * ?__ExceptionPtrToBool@@YA_NPEBX@Z
+ */
+MSVCRT_bool __cdecl __ExceptionPtrToBool(exception_ptr *ep)
+{
+    return !!ep->rec;
+}
+
 void* __cdecl __AdjustPointer(void *obj, const this_ptr_offsets *off)
 {
     return get_this_pointer(off, obj);
 }
+
+#if _MSVCR_VER >= 140
+typedef struct
+{
+    char *name;
+    char mangled[1];
+} type_info140;
+
+typedef struct
+{
+    SLIST_ENTRY entry;
+    char name[1];
+} type_info_entry;
+
+static void* CDECL type_info_entry_malloc(MSVCRT_size_t size)
+{
+    type_info_entry *ret = MSVCRT_malloc(FIELD_OFFSET(type_info_entry, name) + size);
+    return ret->name;
+}
+
+static void CDECL type_info_entry_free(void *ptr)
+{
+    ptr = (char*)ptr - FIELD_OFFSET(type_info_entry, name);
+    MSVCRT_free(ptr);
+}
+
+/******************************************************************
+ *		__std_type_info_compare (UCRTBASE.@)
+ */
+int CDECL MSVCRT_type_info_compare(const type_info140 *l, const type_info140 *r)
+{
+    int ret;
+
+    if (l == r) ret = 0;
+    else ret = strcmp(l->mangled + 1, r->mangled + 1);
+    TRACE("(%p %p) returning %d\n", l, r, ret);
+    return ret;
+}
+
+/******************************************************************
+ *		__std_type_info_name (UCRTBASE.@)
+ */
+const char* CDECL MSVCRT_type_info_name_list(type_info140 *ti, SLIST_HEADER *header)
+{
+      if (!ti->name)
+      {
+          char* name = __unDName(0, ti->mangled + 1, 0,
+                  type_info_entry_malloc, type_info_entry_free, UNDNAME_NO_ARGUMENTS | UNDNAME_32_BIT_DECODE);
+          if (name)
+          {
+              unsigned int len = strlen(name);
+
+              while (len && name[--len] == ' ')
+                  name[len] = '\0';
+
+              if (InterlockedCompareExchangePointer((void**)&ti->name, name, NULL))
+              {
+                  type_info_entry_free(name);
+              }
+              else
+              {
+                  type_info_entry *entry = (type_info_entry*)(name-FIELD_OFFSET(type_info_entry, name));
+                  InterlockedPushEntrySList(header, &entry->entry);
+              }
+          }
+      }
+      TRACE("(%p) returning %s\n", ti, ti->name);
+      return ti->name;
+}
+
+/******************************************************************
+ *		__std_type_info_destroy_list  (UCRTBASE.@)
+ */
+void CDECL MSVCRT_type_info_destroy_list(SLIST_HEADER *header)
+{
+    SLIST_ENTRY *cur, *next;
+
+    TRACE("(%p)\n", header);
+
+    for(cur = InterlockedFlushSList(header); cur; cur = next)
+    {
+        next = cur->Next;
+        MSVCRT_free(cur);
+    }
+}
+#endif
