@@ -62,6 +62,7 @@ typedef struct VideoRendererImpl
     RECT WindowPos;
     LONG VideoWidth;
     LONG VideoHeight;
+    LONG FullScreenMode;
 } VideoRendererImpl;
 
 static inline VideoRendererImpl *impl_from_BaseWindow(BaseWindow *iface)
@@ -184,7 +185,7 @@ static void VideoRenderer_AutoShowWindow(VideoRendererImpl *This)
 
         AdjustWindowRectEx(&This->WindowPos, style, FALSE, style_ex);
 
-        TRACE("WindowPos: %d %d %d %d\n", This->WindowPos.left, This->WindowPos.top, This->WindowPos.right, This->WindowPos.bottom);
+        TRACE("WindowPos: %s\n", wine_dbgstr_rect(&This->WindowPos));
         SetWindowPos(This->baseControlWindow.baseWindow.hWnd, NULL,
             This->WindowPos.left,
             This->WindowPos.top,
@@ -242,8 +243,8 @@ static DWORD VideoRenderer_SendSampleData(VideoRendererImpl* This, LPBYTE data,
         return E_FAIL;
     }
 
-    TRACE("Src Rect: %d %d %d %d\n", This->SourceRect.left, This->SourceRect.top, This->SourceRect.right, This->SourceRect.bottom);
-    TRACE("Dst Rect: %d %d %d %d\n", This->DestRect.left, This->DestRect.top, This->DestRect.right, This->DestRect.bottom);
+    TRACE("Src Rect: %s\n", wine_dbgstr_rect(&This->SourceRect));
+    TRACE("Dst Rect: %s\n", wine_dbgstr_rect(&This->DestRect));
 
     StretchDIBits(This->baseControlWindow.baseWindow.hDC, This->DestRect.left, This->DestRect.top, This->DestRect.right -This->DestRect.left,
                   This->DestRect.bottom - This->DestRect.top, This->SourceRect.left, This->SourceRect.top,
@@ -890,7 +891,12 @@ static HRESULT WINAPI VideoWindow_get_FullScreenMode(IVideoWindow *iface,
 {
     VideoRendererImpl *This = impl_from_IVideoWindow(iface);
 
-    FIXME("(%p/%p)->(%p): stub !!!\n", This, iface, FullScreenMode);
+    TRACE("(%p/%p)->(%p): %d\n", This, iface, FullScreenMode, This->FullScreenMode);
+
+    if (!FullScreenMode)
+        return E_POINTER;
+
+    *FullScreenMode = This->FullScreenMode;
 
     return S_OK;
 }
@@ -918,6 +924,7 @@ static HRESULT WINAPI VideoWindow_put_FullScreenMode(IVideoWindow *iface,
         SetWindowPos(This->baseControlWindow.baseWindow.hWnd,0,This->DestRect.left,This->DestRect.top,This->DestRect.right,This->DestRect.bottom,SWP_NOZORDER|SWP_SHOWWINDOW);
         This->WindowPos = This->DestRect;
     }
+    This->FullScreenMode = FullScreenMode;
 
     return S_OK;
 }
@@ -1025,6 +1032,7 @@ HRESULT VideoRenderer_create(IUnknown *pUnkOuter, void **ppv)
     ZeroMemory(&pVideoRenderer->SourceRect, sizeof(RECT));
     ZeroMemory(&pVideoRenderer->DestRect, sizeof(RECT));
     ZeroMemory(&pVideoRenderer->WindowPos, sizeof(RECT));
+    pVideoRenderer->FullScreenMode = OAFALSE;
 
     if (pUnkOuter)
         pVideoRenderer->outer_unk = pUnkOuter;
