@@ -481,6 +481,7 @@ static int reg_delete(HKEY root, WCHAR *path, WCHAR *key_name, WCHAR *value_name
             }
             else break;
         }
+        HeapFree(GetProcessHeap(), 0, szValue);
     }
     else if (value_name || value_empty)
     {
@@ -582,6 +583,7 @@ static const WCHAR *reg_type_to_wchar(DWORD type)
 static void output_value(const WCHAR *value_name, DWORD type, BYTE *data, DWORD data_size)
 {
     WCHAR fmt[] = {' ',' ',' ',' ','%','1',0};
+    WCHAR defval[32];
     WCHAR *reg_data;
     WCHAR newlineW[] = {'\n',0};
 
@@ -589,14 +591,22 @@ static void output_value(const WCHAR *value_name, DWORD type, BYTE *data, DWORD
         output_string(fmt, value_name);
     else
     {
-        WCHAR defval[32];
         LoadStringW(GetModuleHandleW(NULL), STRING_DEFAULT_VALUE, defval, ARRAY_SIZE(defval));
         output_string(fmt, defval);
     }
     output_string(fmt, reg_type_to_wchar(type));
-    reg_data = reg_data_to_wchar(type, data, data_size);
-    output_string(fmt, reg_data);
-    HeapFree(GetProcessHeap(), 0, reg_data);
+
+    if (data)
+    {
+        reg_data = reg_data_to_wchar(type, data, data_size);
+        output_string(fmt, reg_data);
+        HeapFree(GetProcessHeap(), 0, reg_data);
+    }
+    else
+    {
+        LoadStringW(GetModuleHandleW(NULL), STRING_VALUE_NOT_SET, defval, ARRAY_SIZE(defval));
+        output_string(fmt, defval);
+    }
     output_string(newlineW);
 }
 
@@ -660,8 +670,13 @@ static int query_value(HKEY key, WCHAR *value_name, WCHAR *path, BOOL recurse)
     {
         if (rc == ERROR_FILE_NOT_FOUND)
         {
-            output_message(STRING_CANNOT_FIND);
-            return 1;
+            if (value_name && *value_name)
+            {
+                output_message(STRING_CANNOT_FIND);
+                return 1;
+            }
+            output_string(fmt, path);
+            output_value(NULL, REG_SZ, NULL, 0);
         }
         return 0;
     }
@@ -818,6 +833,53 @@ static int reg_query(HKEY root, WCHAR *path, WCHAR *key_name, WCHAR *value_name,
     return ret;
 }
 
+static WCHAR *get_long_key(HKEY root, WCHAR *path)
+{
+    DWORD i, array_size = ARRAY_SIZE(root_rels), len;
+    WCHAR *long_key;
+    WCHAR fmt[] = {'%','s','\\','%','s',0};
+
+    for (i = 0; i < array_size; i++)
+    {
+        if (root == root_rels[i].key)
+            break;
+    }
+
+    len = strlenW(root_rels[i].long_name);
+
+    if (!path)
+    {
+        long_key = HeapAlloc(GetProcessHeap(), 0, (len + 1) * sizeof(WCHAR));
+        strcpyW(long_key, root_rels[i].long_name);
+        return long_key;
+    }
+
+    len += strlenW(path) + 1; /* add one for the backslash */
+    long_key = HeapAlloc(GetProcessHeap(), 0, (len + 1) * sizeof(WCHAR));
+    sprintfW(long_key, fmt, root_rels[i].long_name, path);
+    return long_key;
+}
+
+static BOOL parse_registry_key(const WCHAR *key, HKEY *root, WCHAR **path, WCHAR **long_key)
+{
+    if (!sane_path(key))
+        return FALSE;
+
+    *root = path_get_rootkey(key);
+    if (!*root)
+    {
+        output_message(STRING_INVALID_KEY);
+        return FALSE;
+    }
+
+    *path = strchrW(key, '\\');
+    if (*path) (*path)++;
+
+    *long_key = get_long_key(*root, *path);
+
+    return TRUE;
+}
+
 static BOOL is_help_switch(const WCHAR *s)
 {
     if (strlenW(s) > 2)
@@ -906,21 +968,9 @@ int wmain(int argc, WCHAR *argvW[])
         return 0;
     }
 
-    key_name = argvW[2];
-
-    if (!sane_path(key_name))
+    if (!parse_registry_key(argvW[2], &root, &path, &key_name))
         return 1;
 
-    root = path_get_rootkey(key_name);
-    if (!root)
-    {
-        output_message(STRING_INVALID_KEY);
-        return 1;
-    }
-
-    path = strchrW(key_name, '\\');
-    if (path) path++;
-
     for (i = 3; i < argc; i++)
     {
         if (argvW[i][0] == '/' || argvW[i][0] == '-')
